package edu.toronto.cs.genmodel.tests;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.internal.resources.Folder;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.codegen.ecore.Generator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.osgi.framework.Bundle;

import junit.framework.TestCase;
import junit.textui.TestRunner;
public class BuildModel extends TestCase {
	public static void main(String[] args) {
		TestRunner.run(BuildModel.class);
	}
	public BuildModel(String name) {
		super(name);
	}
	protected void setUp() throws Exception {
	}
	protected void tearDown() throws Exception {
	}
	
	public void testGeneration() {
		String args [] = new String[6];
		IFile models[] = copy_file_to_workspace("build.models");
		Generator g = new Generator();
		for (int i=0; i<models.length; i++) {
			IFile file = models[i];
			args[0] = "-autoBuild";
			args[1] = "true";
			args[2] = "-model";
			args[3] = "-edit";
			args[4] = "-editor";
	 		args[5] = file.getLocation().toPortableString(); // the genmodel file
			g.run(args);
		}
		g.printGenerateUsage(); // help messages
	}
	private IFile[] copy_file_to_workspace(String dir) {
		ArrayList names = new ArrayList();
		IWorkspace w = ResourcesPlugin.getWorkspace();
		IProject project = w.getRoot().getProject("build.models");
		try {
			Bundle bundle = Activator.getDefault().getBundle();
			URL url = Platform.find(bundle, new Path("model/manifest.txt"));
			url = Platform.resolve(url);
			Object content = url.getContent();
			InputStream ins = (InputStream) content;
			int count = ins.available();
			BufferedReader in = new BufferedReader(new InputStreamReader(ins));
			char[] cbuf = new char[count];
			in.read(cbuf);
			in.close();
			String[] lines = new String(cbuf).split("\n");
			List manifest = new ArrayList();
			for (int i = 0; i < lines.length; i++) {
				String string = lines[i];
				string = string.trim();
				if (string.length() > 0)
					manifest.add(string);
			}
			String errors = "";
			if (!project.exists())
				project.create(null);
			else {
				errors += "The models project already exists." + "\n";
				System.out.println("models" +
						"Errors creating the models project: " + errors);
				return null;
			}
			project.open(null);
			for (Iterator iter = manifest.iterator(); iter.hasNext();) {
				try {
					String name = (String) iter.next();
					String[] words = name.split("/");
					String pathName = "";
					IFolder folder = null;
					for (int i = 0; i < words.length - 1; i++) {
						String string = words[i];
						pathName = pathName + string + "/";
						folder = project.getFolder(pathName);
						if (!folder.exists())
							folder.create(IResource.NONE, true, null);
					}
					IPath path = new Path(name);
					InputStream stream = Activator.getDefault()
							.openStream(path);
					IFile file;
					if (folder == null)
						file = project.getFile(name);
					else
						file = folder.getFile(words[words.length - 1]);						
					file.create(stream, false, null);
					stream.close();
					if (file.getFileExtension().equals("genmodel"))
						names.add(file);
				} catch (CoreException x) {
					errors += x.toString() + "\n";
					x.printStackTrace();
				} catch (IOException x) {
					errors += x.toString() + "\n";
					x.printStackTrace();
				}
			}
			if (errors.length() > 0) {
				System.out.println(
						"models:" + "Errors creating the models project: " + errors);
			}
		} catch (CoreException x) {
			System.out.println(
					"models:" + "Errors creating the models project due to: " + x);
		} catch (IOException x) {
			System.out.println(
					"models:" + "Errors creating the Examples project due to : " + x);
		} catch (Exception x) {
			x.printStackTrace();
		}
		if (names.size()>0) {
			IFile files[] = new IFile[names.size()];
			files = (IFile[])names.toArray(files);			
			return files;
		}
		return null;
	}	
}
